package com.bizmatics.common.mvc.config.convert;

import com.bizmatics.common.core.enums.IEnum;
import com.bizmatics.common.core.util.StringUtils;
import lombok.SneakyThrows;
import org.springframework.core.convert.converter.Converter;
import org.springframework.core.convert.converter.ConverterFactory;

import java.lang.reflect.Method;
import java.util.Map;
import java.util.WeakHashMap;

public class BaseEnumConverterFactory implements ConverterFactory<String, IEnum> {

    private static final Map<Class, Converter> converterMap = new WeakHashMap<>();

    private static final String ENUM_CONVERT_METHOD_NAME = "parse";

    @SneakyThrows
    @Override
    public <T extends IEnum> Converter<String, T> getConverter(Class<T> targetType) {

        Converter result = converterMap.get(targetType);
        if (result == null) {
            result = new StringToIEumConverter<T>(targetType);
            converterMap.put(targetType, result);
        }
        return result;
    }

    private static class StringToIEumConverter<T extends IEnum> implements Converter<String, T> {

        private final Class<T> targerType;

        public StringToIEumConverter(Class<T> targerType) {
            this.targerType = targerType;
        }

        @SneakyThrows
        @Override
        public T convert(String source) {
            if (StringUtils.isEmpty(source)) {
                return null;
            }
            Method method = targerType.getDeclaredMethod(ENUM_CONVERT_METHOD_NAME, String.class);
            Object o = method.invoke(null, source);
            return (T) o;
        }
    }

}
