package com.bizmatics.mhfire.controller.web;

import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.mhfire.persistence.mapper.po.FireStatisticsPO;
import com.bizmatics.mhfire.service.FireStatisticsService;
import com.bizmatics.mhfire.service.vo.FireBubbleVO;
import com.bizmatics.mhfire.service.vo.FireLevelRatioVO;
import com.bizmatics.mhfire.service.vo.FireUnitPoliceVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.xml.crypto.Data;
import java.util.Date;
import java.util.List;

/**
 * 火灾详情
 * @author yq
 * @date 2021/5/31 10:20
 */
@RestController
@RequestMapping("/fireStatistics")
public class FireStatisticsControllerWeb {


    @Autowired
    private FireStatisticsService fireStatisticsService;


    /**
     * 根据年份和地址查看12月份的火灾统计
     * @param startTime 开始时间
     * @param endTime 开始时间
     * @param address 地址
     * @param fireType 火灾类别
     * @return
     */
    @GetMapping("fireCountByData")
    public ApiResult<List<Object>> getFireCountByData(@RequestParam Date startTime,
                                                       @RequestParam Date endTime,
                                                       @RequestParam(required = false)String address,
                                                       @RequestParam(required = false)String fireType){
        return ApiResult.success(fireStatisticsService.getFireCountByData(startTime,endTime,address,fireType));
    }


    /**
     * 分页
     * @param current 页数
     * @param size 条数
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param address 地址
     * @return
     */
    @GetMapping("page")
    public ApiResult<CommonPage<FireStatisticsPO>> page(@RequestParam Integer current,
                                                        @RequestParam Integer size,
                                                        @RequestParam(required = false) Date startTime,
                                                        @RequestParam(required = false) Date endTime,
                                                        @RequestParam(required = false)String address){
        return ApiResult.success(fireStatisticsService.page(current, size, startTime, endTime, address));
    }


    /**
     * getOne
     * @param id
     * @return
     */
    @GetMapping("one")
    public ApiResult<FireStatisticsPO> getOne(@RequestParam String id){
        return ApiResult.success(fireStatisticsService.getOne(id));
    }


    /**
     * 火灾-单位-出警信息
     * @param id
     * @return
     */
    @GetMapping("fireUnitPolice")
    public ApiResult<FireUnitPoliceVO> getFireUnitPolice(@RequestParam String id){
        return ApiResult.success(fireStatisticsService.getFireUnitPolice(id));
    }


    /**
     * 火灾原因占比
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @param address   地址
     * @return
     */
    @GetMapping("fireCauseRatio")
    public ApiResult<List<FireLevelRatioVO>> getGroupByLevel(@RequestParam(required = false) Date startTime,
                                                             @RequestParam(required = false)Date endTime,
                                                             @RequestParam(required = false)String address){
        return ApiResult.success(fireStatisticsService.getGroupByLevel(startTime,endTime,address));
    }

    /**
     * 火灾类型
     * @return
     */
    @GetMapping("fireType")
    public ApiResult<List<String>> getFireType(){
        return ApiResult.success(fireStatisticsService.getFireType());
    }

    @GetMapping("fireAddress")
    public ApiResult<List<String>> getAddress(){
        return ApiResult.success(fireStatisticsService.getAddress());
    }

    @GetMapping("fireBubble")
    public ApiResult<List<FireBubbleVO>> getFireBubble(@RequestParam(required = false)Date startTime,
                                                       @RequestParam(required = false)Date endTime,
                                                       @RequestParam(required = false)String fireType,
                                                       @RequestParam(required = false)String unitId){
        return ApiResult.success(fireStatisticsService.getAvgAndSum(startTime, endTime, fireType, unitId));
    }
}
