package com.bizmatics.mhfire.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.common.core.util.DateUtils;
import com.bizmatics.common.core.util.StringUtils;
import com.bizmatics.mhfire.persistence.mapper.FireStatisticsMapper;
import com.bizmatics.mhfire.persistence.mapper.po.FireStatisticsPO;
import com.bizmatics.mhfire.persistence.mapper.po.UnitPO;
import com.bizmatics.mhfire.persistence.mapper.vo.LocateInfo;
import com.bizmatics.mhfire.service.FireStatisticsService;
import com.bizmatics.mhfire.service.UnitService;
import com.bizmatics.mhfire.service.util.Arith;
import com.bizmatics.mhfire.service.util.GouldUtil;
import com.bizmatics.mhfire.service.vo.FireBubbleVO;
import com.bizmatics.mhfire.service.vo.FireLevelRatioVO;
import com.bizmatics.mhfire.service.vo.FireUnitPoliceVO;
import com.google.common.collect.Lists;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author yq
 * @date 2021/5/28 9:35
 */
@Service
public class FireStatisticsServiceImpl implements FireStatisticsService {

    @Autowired
    private FireStatisticsMapper fireStatisticsMapper;
    @Autowired
    private UnitService unitService;
    @Override
    public List<Object> getFireCountByData(Date startTime,Date endTime,String address,String fireType) {
        List<Object> list = new ArrayList<>();
        Date date = new Date();
        Date beginDayOfYear = DateUtils.getBeginDayOfYear(date);
        //获取传入时间的最大时间和最小时间
        int distanceOfTwoDate =Integer.parseInt(DateUtils.getYear(endTime)) - Integer.parseInt(DateUtils.getYear(startTime)) + 1;
        //获取数据库全部项目的最小时间和最大时间
        Date maxDate = fireStatisticsMapper.selectMaxDate();
        Date minDate = fireStatisticsMapper.selectMinDate();
        int distanceOfTwoDateYear =Integer.parseInt(DateUtils.getYear(maxDate)) - Integer.parseInt(DateUtils.getYear(minDate)) + 1;
        list.add(perfect(beginDayOfYear, date, address, fireType, 1));
        list.add(perfect(startTime,endTime,address,fireType,distanceOfTwoDate));
        list.add(perfect(null,null,address,fireType,distanceOfTwoDateYear));
        return list;
    }

    public List<Object> perfect(Date startTime,Date endTime,String address,String fireType, Integer subTime){
        List<Map<String, Object>> list = fireStatisticsMapper.selectCountByDate(startTime, endTime, fireType, address);
        List<Object> avgList = new ArrayList<>();
        for (int i = 1; i <= 12; i++) {
            int finalI = i;
            list.stream()
                    .filter(map -> Integer.parseInt(map.get("monthTime").toString()) == finalI)
                    .findFirst()
                    .map(map -> {
                        avgList.add(Arith.div(Double.parseDouble(map.get("aCount").toString()),subTime,2));
                        return Optional.empty();
                    })
                    .orElseGet(() -> {
                        avgList.add(0.00);
                        return Optional.empty();
                    });
        }
        return avgList;
    }

    @Override
    public CommonPage<FireStatisticsPO> page(Integer current, Integer size, Date startTime, Date endTime, String address) {
        Page<Map<String, String>> page = new Page<>(current, size);
        page = fireStatisticsMapper.page(page, startTime, endTime, address);
        List<FireStatisticsPO> list = page.getRecords()
                .stream()
                .map(this::enhanceFireStatisticsPo).collect(Collectors.toList());
        //筛选出精度和维度是空的数据
        List<FireStatisticsPO> isNullData = list.stream()
                .filter(fireStatisticsPo -> 0.00 == fireStatisticsPo.getLatitude())
                .filter(fireStatisticsPo -> StringUtils.isNotBlank(fireStatisticsPo.getAddress()))
                .collect(Collectors.toList());
        List<List<FireStatisticsPO>> subSets = Lists.partition(isNullData, 10);
        for (List<FireStatisticsPO> listLimit:subSets) {
            String addressList = listLimit.stream().map(FireStatisticsPO::getAddress).collect(Collectors.joining("|"));
            List<LocateInfo> lonLat = GouldUtil.getLonLat(addressList);
            if (CollectionUtils.isNotEmpty(lonLat)){
                for (FireStatisticsPO fireStatisticsPo:listLimit) {
                    fireStatisticsPo.setLatitude(lonLat.get(listLimit.indexOf(fireStatisticsPo)).getLatitude());
                    fireStatisticsPo.setLongitude(lonLat.get(listLimit.indexOf(fireStatisticsPo)).getLongitude());
                    fireStatisticsPo.setDistrict(lonLat.get(listLimit.indexOf(fireStatisticsPo)).getDistrict());
                    fireStatisticsPo.setStreet(lonLat.get(listLimit.indexOf(fireStatisticsPo)).getStreet());

                }
                fireStatisticsMapper.updateList(listLimit);
            }
        }
        return new CommonPage<>(list, page.getTotal(), page.getSize(), page.getCurrent());
    }

    @Override
    public List<FireStatisticsPO> list(Date startTime, Date endTime, String address) {
        return fireStatisticsMapper.list(startTime,endTime,address)
                .stream()
                .map(this::enhanceFireStatisticsPo).collect(Collectors.toList());
    }

    @Override
    public List<FireStatisticsPO> distanceList(Double longitude, Double latitude, Date startTime, Date endTime) {
        return fireStatisticsMapper.distanceList(longitude, latitude, startTime, endTime)
                .stream()
                .map(this::enhanceFireStatisticsPo).collect(Collectors.toList());
    }

    @Override
    public FireStatisticsPO getOne(String id) {
        return enhanceFireStatisticsPo(fireStatisticsMapper.getOne(id));
    }

    @Override
    public FireUnitPoliceVO getFireUnitPolice(String id) {
        //火灾详细信息
        FireStatisticsPO fireStatisticsPo = getOne(id);
        //单位信息
        UnitPO unitPo = new UnitPO();
        unitPo.setAddress(fireStatisticsPo.getAddress());
        unitPo = unitService.getOne(unitPo);

        //TODO 出警记录没有
        FireUnitPoliceVO fireUnitPoliceVO = new FireUnitPoliceVO();
        fireUnitPoliceVO.setFireStatisticsPo(fireStatisticsPo);
        fireUnitPoliceVO.setUnitPo(unitPo);
        return fireUnitPoliceVO;
    }

    @Override
    public List<FireLevelRatioVO> getGroupByLevel(Date startTime, Date endTime, String address) {
        List<FireLevelRatioVO> list = new ArrayList<>();
        int fireCount = 0;
        List<String> fireCauses = fireStatisticsMapper.groupByCause();
        for (String cause:fireCauses) {
            if (StringUtils.isBlank(cause)){
                continue;
            }
            FireLevelRatioVO fireLevelRatioVo = new FireLevelRatioVO();
            fireLevelRatioVo.setFireType(cause);
            Integer count = fireStatisticsMapper.selectCount(startTime, endTime, address, cause,null,null);
            fireLevelRatioVo.setRadio(count.doubleValue());
            list.add(fireLevelRatioVo);
            fireCount+=count;
        }
        //排序
        List<FireLevelRatioVO> orderByList = list.stream().sorted(Comparator.comparing(FireLevelRatioVO::getRadio).reversed()).limit(10).collect(Collectors.toList());

        //差集求和
        double sum = list.stream().filter(item -> !orderByList.contains(item)).mapToDouble(FireLevelRatioVO::getRadio).sum();
        orderByList.stream().filter(item -> "其他".equals(item.getFireType())).findAny()
                .map(fireLevelRatio -> {
                            fireLevelRatio.setRadio(fireLevelRatio.getRadio() + sum);
                            return fireLevelRatio;
                        }
                ).orElseGet(() -> {FireLevelRatioVO fireLevelRatioVo = new FireLevelRatioVO();
                    fireLevelRatioVo.setFireType("其他");
                    fireLevelRatioVo.setRadio(sum);
                    orderByList.add(fireLevelRatioVo);
                    return fireLevelRatioVo;
                });
        int finalFireCount = fireCount;
        if (0 != finalFireCount){
            orderByList.forEach(fireLevelRatio -> fireLevelRatio.setRadio(Arith.div(fireLevelRatio.getRadio(),finalFireCount)));
        }
        return orderByList;
    }


    @Override
    public List<String> getFireType() {
        List<FireLevelRatioVO> list = new ArrayList<>();
        List<String> fireTypes = fireStatisticsMapper.groupByCause().stream().filter(StringUtils::isNotBlank).collect(Collectors.toList());
        for (String cause:fireTypes) {
            FireLevelRatioVO fireLevelRatioVo = new FireLevelRatioVO();
            fireLevelRatioVo.setFireType(cause);
            Integer count = fireStatisticsMapper.selectCount(null, null,null, cause,null,null);
            fireLevelRatioVo.setRadio(count.doubleValue());
            list.add(fireLevelRatioVo);
        }
        return list.stream().sorted(Comparator.comparing(FireLevelRatioVO::getRadio).reversed())
                .map(FireLevelRatioVO::getFireType).collect(Collectors.toList());

    }

    @Override
    public List<String> getAddress() {
        List<FireLevelRatioVO> list = new ArrayList<>();
        List<String> lists = fireStatisticsMapper.getAddress().stream().filter(StringUtils::isNotBlank).filter(s -> !s.contains("派出所")).collect(Collectors.toList());
        for (String address:lists) {
            FireLevelRatioVO fireLevelRatioVo = new FireLevelRatioVO();
            fireLevelRatioVo.setFireType(address);
            Integer count = fireStatisticsMapper.selectCount(null, null,null, null,null,address);
            fireLevelRatioVo.setRadio(count.doubleValue());
            list.add(fireLevelRatioVo);
        }
        return list.stream().sorted(Comparator.comparing(FireLevelRatioVO::getRadio).reversed())
                .map(FireLevelRatioVO::getFireType).collect(Collectors.toList());
    }

    @Override
    public List<FireBubbleVO> getAvgAndSum(Date startTime, Date endTime,String fireType, String unitId) {
        List<FireBubbleVO> list = new ArrayList<>();
        List<String> lists = fireStatisticsMapper.getAddress().stream().filter(StringUtils::isNotBlank).filter(s -> !s.contains("派出所")).collect(Collectors.toList());
        lists.forEach(s -> {
            Map<String, Double> maps = fireStatisticsMapper.selectAvgAndSum(startTime, endTime, s, fireType, unitId);
            FireBubbleVO fireBubbleVo = new FireBubbleVO();
            fireBubbleVo.setRegion(s);
            fireBubbleVo.setFireCount(fireStatisticsMapper.selectCount(startTime,endTime,null,fireType,unitId,s));
            if (null != maps){
                fireBubbleVo.setDeathToll(Optional.ofNullable(maps.get("deathPerson")).map(Double::intValue).orElse(0));
                fireBubbleVo.setFireArea(Arith.div(maps.get("fireArea"),1));
            }else {
                fireBubbleVo.setDeathToll(0);
                fireBubbleVo.setFireArea(0.00);
            }
            list.add(fireBubbleVo);
        });
        return list;
    }

    public FireStatisticsPO enhanceFireStatisticsPo(Map<String,String> map){FireStatisticsPO fireStatisticsPo = new FireStatisticsPO();
        fireStatisticsPo.setId(Optional.ofNullable(map.get("火灾序号")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setAddress(Optional.ofNullable(map.get("起火地点")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setBurnedArea(Optional.ofNullable(map.get("过火面积")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setPropertyLoss(Optional.ofNullable(map.get("直接财产损失")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setDeathToll(Optional.ofNullable(map.get("死亡人数")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setNonFatal(Optional.ofNullable(map.get("受灾人数")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setDisasterHome(Optional.ofNullable(map.get("受灾户数")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setFireCause(Optional.ofNullable(map.get("火灾原因分类（一级）")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setFireLevel(Optional.ofNullable(map.get("火灾等级")).filter(s -> !"null".equals(s)).orElse(""));
        fireStatisticsPo.setLongitude(Double.valueOf(Optional.ofNullable(map.get("longitude")).filter(StringUtils::isNotBlank).orElse("0.00")));
        fireStatisticsPo.setLatitude(Double.valueOf(Optional.ofNullable(map.get("latitude")).filter(StringUtils::isNotBlank).orElse("0.00")));
        fireStatisticsPo.setDistrict(map.get("district"));
        fireStatisticsPo.setStreet(map.get("street"));
        fireStatisticsPo.setPlaceOne(map.get("起火物（一级）"));
        fireStatisticsPo.setPlaceTwo(map.get("起火物（二级）"));
        fireStatisticsPo.setFireGoodsOne(map.get("起火场所（一级）"));
        fireStatisticsPo.setFireGoodsTwo(map.get("起火场所（二级）"));
        fireStatisticsPo.setNature(map.get("建筑使用性质名称"));
        return fireStatisticsPo;
    }
}
