package com.bizmatics.mhfire.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.mhfire.persistence.mapper.FireWaterMapper;
import com.bizmatics.mhfire.persistence.mapper.po.FireWaterPO;
import com.bizmatics.mhfire.service.FireWaterService;

import com.bizmatics.mhfire.service.util.Arith;
import com.bizmatics.mhfire.service.vo.FireWaterStatisticsVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author yq
 * @date 2021/5/27 16:16
 */
@Service
public class FireWaterServiceImpl implements FireWaterService {

    @Autowired
    private FireWaterMapper fireWaterMapper;
    @Override
    public FireWaterPO getOne(String id) {
        return enhanceFireWaterPo(fireWaterMapper.selectOne(id));
    }

    @Override
    public CommonPage<FireWaterPO> page(Integer current, Integer size, Date startTime, Date endTime) {
        List<FireWaterPO> list = new ArrayList<>();
        Page<Map<String, String>> page = new Page<>(current, size);
        page = fireWaterMapper.page(page, startTime, endTime);
        page.getRecords().forEach(stringObjectMap -> list.add(enhanceFireWaterPo(stringObjectMap)));
        return new CommonPage<>(list, page.getTotal(), page.getSize(), page.getCurrent());
    }

    @Override
    public FireWaterStatisticsVO getFireWaterStatistics(Date startTime, Date endTime) {
        FireWaterStatisticsVO fireWaterStatisticsVO = new FireWaterStatisticsVO();
        fireWaterStatisticsVO.setMunicipalFireCode(
                fireWaterMapper.selectCountByWaterNature("消火栓","市政",null,startTime,endTime,null));
        fireWaterStatisticsVO.setCommunityFireCode(
                fireWaterMapper.selectCountByWaterNature("消火栓","居民",null,startTime,endTime,null));
        fireWaterStatisticsVO.setUnitFireCode(
                fireWaterMapper.selectCountByWaterNature("消火栓","单位",null,startTime,endTime,null));
        fireWaterStatisticsVO.setNaturalWater(
                fireWaterMapper.selectCountByWaterNature("天然水源",null,null,startTime,endTime,null));
        fireWaterStatisticsVO.setMunicipalWater(
                fireWaterMapper.selectCountByWaterNature(null,"市政",null,startTime,endTime,null));
        fireWaterStatisticsVO.setResidentWater(
                fireWaterMapper.selectCountByWaterNature(null,"居民",null,startTime,endTime,null));
        fireWaterStatisticsVO.setUnitWater(
                fireWaterMapper.selectCountByWaterNature(null,"单位",null,startTime,endTime,null));
        fireWaterStatisticsVO.setMunicipalGetWater(
                fireWaterMapper.selectCountByWaterNature(null,null,"市政",startTime,endTime,null));
        fireWaterStatisticsVO.setLandGetWater(
                fireWaterMapper.selectCountByWaterNature(null,null,"地上",startTime,endTime,null));
        Integer total =fireWaterMapper.selectCountByWaterNature(null,null,null,startTime,endTime,null);
        fireWaterStatisticsVO.setMunicipalGetWaterRadio(Arith.div(fireWaterStatisticsVO.getMunicipalGetWater(),total));
        fireWaterStatisticsVO.setLandGetWaterRadio(Arith.div(fireWaterStatisticsVO.getLandGetWater(),total));
        Integer waterIntact = fireWaterMapper.selectCountByWaterNature(null, null, null, startTime, endTime, "1");
        fireWaterStatisticsVO.setWaterIntact(Arith.div(waterIntact,total));
        return fireWaterStatisticsVO;
    }

    /**
     * map转换为unitpo
     * @param fireWaterMap
     * @return
     */
    public FireWaterPO enhanceFireWaterPo(Map<String,String> fireWaterMap){
        FireWaterPO fireWaterPo = new FireWaterPO();
        fireWaterPo.setAddress(fireWaterMap.get("水源地址"));
        fireWaterPo.setPhone(Optional.ofNullable(fireWaterMap.get("联系方式")).orElse(""));
        fireWaterPo.setType(fireWaterMap.get("水源类型"));
        fireWaterPo.setJurisdictionalAgency(fireWaterMap.get("管辖机构名称"));
        fireWaterPo.setDetachmentName(fireWaterMap.get("支队名称"));
        fireWaterPo.setGetWaterType(fireWaterMap.get("取水形式"));
        fireWaterPo.setWaterAdministrative(Optional.ofNullable(fireWaterMap.get("水源性质")).orElse(""));
        fireWaterPo.setStatus(fireWaterMap.get("可用状态"));
        fireWaterPo.setWaterUnit(fireWaterMap.get("供水单位"));
        fireWaterPo.setWaterGage(fireWaterMap.get("管网压力"));
        fireWaterPo.setGisX(fireWaterMap.get("GIS_X"));
        fireWaterPo.setGisY(fireWaterMap.get("GIS_Y"));
        fireWaterPo.setGisBaiDuX(fireWaterMap.get("GIS_X百度坐标"));
        fireWaterPo.setGisBaiDuY(fireWaterMap.get("GIS_Y百度坐标"));
        return fireWaterPo;
    }
}
