package com.bizmatics.mhfire.service.job;

import com.bizmatics.common.core.util.DateUtils;
import com.bizmatics.common.core.util.StringUtils;
import com.bizmatics.mhfire.model.Alert;
import com.bizmatics.mhfire.service.AlertService;
import com.bizmatics.mhfire.service.api.AlertAndSiteApi;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author yq
 * @date 2021/8/12 16:04
 */
@Slf4j
@Component
public class AlertJob{

    @Autowired
    private AlertService alertService;


    //    @Scheduled(cron = "0 0 23 * * ?")
    public void execute(){
        try {
            String token = AlertAndSiteApi.login(AlertAndSiteApi.USER_NAME, AlertAndSiteApi.USER_PASSWORD);
            int page = 1;
            int size = 100;
            executeCore(page,size,null,null,token);
        }catch (Exception e){
            log.info("alertJob--all--异常"+e.getMessage());
        }
    }

    //    @Scheduled(cron = "*/30 * * * * ?")
    public void executeToday(){
        try {
            String token = AlertAndSiteApi.login(AlertAndSiteApi.USER_NAME, AlertAndSiteApi.USER_PASSWORD);
            int page = 1;
            int size = 100;
            Date date = new Date();
            Date dayStartTime = DateUtils.getDayStartTime(date);
            Date dayEndTime = DateUtils.getDayEndTime(date);
            executeCore(page,size,dayStartTime,dayEndTime,token);
        }catch (Exception e){
            log.info("alertJob--today--异常"+e.getMessage());
        }
    }

    public void executeCore(Integer page,Integer size,Date startTime,Date endTime,String token){
        boolean nextFlag = true;
        int count = 0;
        while (StringUtils.isNotBlank(token) && nextFlag){
            try {
                long l = System.currentTimeMillis();
                List<Alert> alerts = AlertAndSiteApi.alertPage(null, null, page, size, token);
                long ea = System.currentTimeMillis();
                log.info("alertJob----调用api需要时间"+(ea-l)+"毫秒");
                if (null != startTime && null != endTime){
                    alerts = alerts.stream().filter(alert -> DateUtils.isEffectiveDate(alert.getDcsj(), startTime, endTime)).collect(Collectors.toList());
                }
                count+=alerts.size();
                if (CollectionUtils.isNotEmpty(alerts)){
                    alertService.saveOrUpdateBatch(alerts);
                    page++;
                }else {
                    nextFlag = false;
                }
                long e = System.currentTimeMillis();
                log.info("alertJob----处理100条数需要的时间"+(e-l)+"毫秒");
            }catch (Exception e){
                log.info("alertJob----异常"+e.getMessage());
            }
        }
        log.info("alertJob----完成,获取警情记录:"+ count);
    }
}
